@@ -2,10 +2,8 @@ import functools
 import importlib
 import os
 import re
-import runpy
 import sys
 from distutils.core import run_setup
-from itertools import chain
 from time import time
 from types import ModuleType
 from typing import (
@@ -21,7 +19,7 @@ from typing import (
     Union,
 )
 
-from .utilities import run
+from .utilities import run, run_module_as_main
 
 lru_cache: Callable[..., Any] = functools.lru_cache
 
@@ -57,12 +55,12 @@ def _list_dist(root: str, modified_at_or_after: float = 0.0) -> FrozenSet[str]:
 
 def _setup(root: str) -> FrozenSet[str]:
     start_time: float = time()
-    current_directory: str = os.path.curdir
+    current_directory: str = os.path.abspath(os.path.curdir)
     os.chdir(root)
-    abs_setup: str = os.path.join(root, "setup.py")
-    setup_args: List[str] = ["sdist", "bdist_wheel"]
-    print(f'{sys.executable} {abs_setup} {" ".join(setup_args)}')
     try:
+        abs_setup: str = os.path.join(root, "setup.py")
+        setup_args: List[str] = ["sdist", "bdist_wheel"]
+        print(f'{sys.executable} {abs_setup} {" ".join(setup_args)}')
         run_setup(abs_setup, setup_args)
     finally:
         os.chdir(current_directory)
@@ -129,24 +127,16 @@ def _get_credentials_from_cerberus() -> Tuple[Optional[str], Optional[str]]:
 
 
 def _dist(root: str, distributions: FrozenSet[str], echo: bool = True) -> None:
-    argv: List[str] = sys.argv
-    twine_argv: List[str] = list(
-        chain(sys.argv[:1], ["upload"], sys.argv[1:], sorted(distributions))
+    run_module_as_main(
+        "twine",
+        arguments=(["upload"] + sys.argv[1:] + list(sorted(distributions))),
+        directory=root,
+        echo=False,
     )
-    current_directory: str = os.path.curdir
-    os.chdir(root)
-    if echo:
-        print(" ".join(["twine"] + twine_argv[1:]))  # type: ignore
-    try:
-        sys.argv = twine_argv
-        runpy.run_module("twine", run_name="__main__")
-    finally:
-        os.chdir(current_directory)
-        sys.argv = argv
 
 
 def _cleanup(root: str) -> None:
-    current_directory: str = os.path.curdir
+    current_directory: str = os.path.abspath(os.path.curdir)
     os.chdir(root)
     try:
         run_setup(os.path.join(root, "setup.py"), ["clean", "--all"])
