@@ -5,7 +5,7 @@ from typing import Iterable, IO
 from typing import Tuple
 from configparser import ConfigParser
 from .requirements.utilities import (
-    setup_egg_info,
+    setup_dist_egg_info,
     iter_distribution_location_file_paths,
 )
 
@@ -25,7 +25,7 @@ def _get_project_and_setup_cfg_paths(path: str = ".") -> Tuple[str, str]:
 
 def _touch_packages_py_typed(project_path: str) -> Iterable[str]:
     # Refresh package metadata
-    setup_egg_info(project_path)
+    setup_dist_egg_info(project_path)
 
     def touch_py_typed(path: str) -> str:
         if os.path.basename(path).lower() == "__init__.py":
