@@ -6,16 +6,29 @@ from io import StringIO
 from dataclasses import dataclass
 from pkg_resources import Distribution
 from configparser import ConfigParser, SectionProxy
-from typing import Dict, Iterable, IO, List, Callable, Set, Union, Tuple, Any
+from typing import (
+    Dict,
+    Iterable,
+    IO,
+    List,
+    Callable,
+    Set,
+    Union,
+    Tuple,
+    Any,
+    Optional,
+)
 from packaging.specifiers import Specifier, SpecifierSet
 from packaging.requirements import Requirement
 from packaging.version import Version, LegacyVersion, parse as parse_version
 from more_itertools import unique_everseen
 from ..utilities import iter_parse_delimited_values
 from .utilities import (
+    is_editable,
     normalize_name,
     get_installed_distributions,
     is_requirement_string,
+    setup_dist_egg_info,
 )
 
 
@@ -127,12 +140,16 @@ def _get_updated_requirement_string(
     name: str = normalize_name(requirement.name)
     if name in ignore:
         return requirement_string
+    distribution: Optional[Distribution] = None
     try:
-        distribution: Distribution = get_installed_distributions()[name]
-        _update_requirement_specifiers(requirement, distribution.version)
+        distribution = get_installed_distributions()[name]
     except KeyError:
         # If the requirement isn't installed, we can't update the version
         pass
+    if distribution is not None:
+        if is_editable(name):
+            setup_dist_egg_info(distribution.location)
+        _update_requirement_specifiers(requirement, distribution.version)
     return str(requirement)
 
 
@@ -242,7 +259,15 @@ def get_updated_tox_ini(data: str, ignore: Iterable[str] = ()) -> str:
     ignore_set: Set[str] = _normalize_ignore_argument(ignore)
 
     def get_updated_requirement_string(requirement: str) -> str:
-        return _get_updated_requirement_string(requirement, ignore=ignore_set)
+        prefix: Optional[str] = None
+        if ":" in requirement:
+            prefix, requirement = requirement.split(":", maxsplit=1)
+        requirement = _get_updated_requirement_string(
+            requirement, ignore=ignore_set
+        )
+        if prefix is not None:
+            requirement = f"{prefix}: {requirement.lstrip()}"
+        return requirement
 
     # Parse
     parser: ConfigParser = ConfigParser()
