@@ -3,6 +3,7 @@ import os
 import tomli
 import pkg_resources
 import importlib_metadata
+from collections import deque
 from warnings import warn
 from pipes import quote
 from configparser import ConfigParser, SectionProxy
@@ -24,7 +25,9 @@ from packaging.utils import canonicalize_name
 from packaging.requirements import InvalidRequirement, Requirement
 from more_itertools import unique_everseen
 from ..utilities import lru_cache, run
+from ..errors import append_exception_text
 
+_BUILTIN_DISTRIBUTION_NAMES: Tuple[str] = ("distribute",)
 # This variable tracks the absolute file paths from which a package has been
 # re-installed, in order to avoid performing a reinstall redundantly
 _reinstalled_locations: Set[str] = set()
@@ -242,167 +245,28 @@ def _distribution_is_editable(
     return any(map(project_egg_link_exists, sys.path))
 
 
-def _iter_editable_distributions(
-    include: Set[str],
-    exclude: Set[str],
-    include_locations: Set[str],
-    exclude_locations: Set[str],
-) -> Iterable[pkg_resources.Distribution]:
-    def include_distribution_item(
-        name_distribution: Tuple[str, pkg_resources.Distribution]
-    ) -> bool:
-        name: str
-        distribution: pkg_resources.Distribution
-        name, distribution = name_distribution
-        if (
-            ((not include) or (name in include))
-            and ((not exclude) or (name not in exclude))
-            and (
-                (not include_locations)
-                or (
-                    os.path.abspath(distribution.location)
-                    not in include_locations
-                )
-            )
-            and (
-                (not exclude_locations)
-                or (
-                    os.path.abspath(distribution.location)
-                    not in exclude_locations
-                )
-            )
-        ):
-            return _distribution_is_editable(distribution)
-        return False
-
-    return map(
-        list.pop,  # type: ignore
-        map(
-            list,
-            filter(
-                include_distribution_item,
-                get_installed_distributions().items(),
-            ),
-        ),
-    )
-
-
-def _reinstall_editable_distribution(
-    distribution: pkg_resources.Distribution, echo: bool = False
-) -> bool:
-    return _reinstall_location(distribution.location, echo=echo)
-
-
-def _reinstall_location(
-    location: str, user: bool = True, echo: bool = False
-) -> bool:
-    user_flag = "--user " if user else ""
-    try:
-        run(
-            (
-                f"{quote(sys.executable)} -m pip install {user_flag}--no-deps "
-                f"-e {quote(location)}"
-            ),
-            echo=echo,
-        )
-        _reinstalled_locations.add(os.path.abspath(location))
-        return True
-    except OSError:
-        if user:
-            return _reinstall_location(location, user=False, echo=echo)
-        else:
-            # If an error code is returned, we just assume package metadata is
-            # up-to-date
-            return False
-
-
-def reinstall_location(location: str, echo: bool = False) -> str:
-    """
-    Re-install a distribution at the local file path `location`,
-    and return the distribution name, or "" if no distribution could
-    be found
-    """
-    if _reinstall_location(location, echo):
-        return _get_location_distribution_name(location)
-    return ""
-
-
-def reinstall_editable(
-    include: Iterable[str] = (),
-    exclude: Iterable[str] = (),
-    include_locations: Iterable[str] = (),
-    exclude_locations: Iterable[str] = (),
-    echo: bool = False,
-) -> None:
-    """
-    This function re-installs editable distributions.
-
-    Parameters:
-
-    - include ([str]):
-      One or more distribution names to include (excluding all others)
-    - exclude ([str])
-      One or more distribution names to exclude
-    - include_locations ([str])
-      One or more distribution locations to include (excluding all others)
-    - exclude_locations ([str])
-      One or more distribution locations to exclude
-    - echo (bool): If `True`, the "pip install ..." commands are printed to
-      `sys.stdout`
-    """
-    if isinstance(include, str):
-        include = {normalize_name(include)}
-    else:
-        include = set(map(normalize_name, include))
-    if isinstance(exclude, str):
-        exclude = {normalize_name(exclude)}
-    else:
-        exclude = set(map(normalize_name, exclude))
-    if isinstance(include_locations, str):
-        include_locations = {os.path.abspath(include_locations)}
-    else:
-        include_locations = set(map(os.path.abspath, include_locations))
-    if isinstance(exclude_locations, str):
-        exclude_locations = {os.path.abspath(exclude_locations)}
-    else:
-        exclude_locations = set(map(os.path.abspath, exclude_locations))
-    # Don't re-install a location more than once
-    exclude_locations |= _reinstalled_locations
-
-    def reinstall_distribution_(
-        distribution: pkg_resources.Distribution,
-    ) -> None:
-        _reinstall_editable_distribution(distribution, echo=echo)
-
-    list(
-        map(
-            reinstall_distribution_,
-            _iter_editable_distributions(
-                include, exclude, include_locations, exclude_locations
-            ),
-        )
-    )
-    refresh_working_set()
-
-
 def _get_setup_py_distribution_name(path: str) -> str:
-    current_directory: str = os.curdir
-    if os.path.basename(path).lower() == "setup.py":
-        os.chdir(os.path.dirname(path))
-    else:
-        if not os.path.isdir(path):
-            path = os.path.dirname(path)
-        os.chdir(path)
-        path = os.path.join(path, "setup.py")
+    current_directory: str = os.path.abspath(os.curdir)
     try:
-        name: str = (
-            run(f"{quote(sys.executable)} {quote(path)} --name", echo=False)
-            .strip()
-            .split("\n")[-1]
-        )
-    except OSError:
-        name = ""
-    os.chdir(current_directory)
+        if os.path.basename(path).lower() == "setup.py":
+            os.chdir(os.path.dirname(path))
+        else:
+            if not os.path.isdir(path):
+                path = os.path.dirname(path)
+            os.chdir(path)
+            path = os.path.join(path, "setup.py")
+        try:
+            name: str = (
+                run(
+                    f"{quote(sys.executable)} {quote(path)} --name", echo=False
+                )
+                .strip()
+                .split("\n")[-1]
+            )
+        except OSError:
+            name = ""
+    finally:
+        os.chdir(current_directory)
     return name
 
 
@@ -429,17 +293,47 @@ def get_setup_distribution_name(path: str) -> str:
     )
 
 
-def _setup_egg_info(location: str) -> None:
+def _setup(arguments: str) -> None:
+    run(
+        f"{quote(sys.executable)} setup.py {arguments}",
+        echo=False,
+    )
+
+
+def _setup_location(location: str, arguments: Iterable[str]) -> None:
     # If there is no setup.py file, we can't update egg info
     if not os.path.isfile(os.path.join(location, "setup.py")):
         return
-    current_directory: str = os.curdir
+    if isinstance(arguments, str):
+        arguments = (arguments,)
+    current_directory: str = os.path.abspath(os.curdir)
     os.chdir(location)
-    run(
-        f"{quote(sys.executable)} setup.py -q egg_info",
-        echo=False,
-    )
-    os.chdir(current_directory)
+    try:
+        deque(map(_setup, arguments), maxlen=0)
+    finally:
+        os.chdir(current_directory)
+
+
+def setup_dist_egg_info(directory: str) -> None:
+    """
+    Refresh dist-info and egg-info for the editable package installed in
+    `directory`
+    """
+    directory = os.path.abspath(directory)
+    if not os.path.isdir(directory):
+        directory = os.path.dirname(directory)
+    return _setup_location(directory, ("-q dist_info", "-q egg_info"))
+
+
+def setup_dist_info(directory: str) -> None:
+    """
+    Refresh dist-info and egg-info for the editable package installed in
+    `directory`
+    """
+    directory = os.path.abspath(directory)
+    if not os.path.isdir(directory):
+        directory = os.path.dirname(directory)
+    return _setup_location(directory, "-q dist_info")
 
 
 def setup_egg_info(directory: str) -> None:
@@ -450,7 +344,7 @@ def setup_egg_info(directory: str) -> None:
     directory = os.path.abspath(directory)
     if not os.path.isdir(directory):
         directory = os.path.dirname(directory)
-    return _setup_egg_info(directory)
+    return _setup_location(directory, "-q egg_info")
 
 
 def get_setup_distribution_requirements(
@@ -470,7 +364,7 @@ def _get_location_distribution_name(location: str) -> str:
     Get a distribution name based on an installation location, or return
     an empty string if no distribution can be found
     """
-    _setup_egg_info(location)
+    setup_dist_egg_info(location)
     location = os.path.abspath(location)
 
     def _is_in_location(
@@ -592,11 +486,16 @@ def install_requirement(
     - echo (bool) = True: If `True` (default), the `pip install`
       commands will be echoed to `sys.stdout`
     """
-    requirement_string: str = (
-        requirement if isinstance(requirement, str) else str(requirement)
-    )
     if isinstance(requirement, str):
         requirement = Requirement(requirement)
+    return _install_requirement(requirement, echo)
+
+
+def _install_requirement(
+    requirement: Union[Requirement, pkg_resources.Requirement],
+    echo: bool = True,
+) -> None:
+    requirement_string: str = str(requirement)
     # Get the distribution name
     name: str = normalize_name(
         requirement.name
@@ -614,12 +513,12 @@ def install_requirement(
     # the editable location
     if distribution and editable:
         # Refresh metadata
-        setup_egg_info(distribution.location)
+        setup_dist_egg_info(distribution.location)
         # Assemble a requirement specifier for the editable install
         requirement_string = distribution.location
         if requirement.extras:
             requirement_string = (
-                f"{requirement_string}" f"[{','.join(requirement.extras)}]"
+                f"{requirement_string}[{','.join(requirement.extras)}]"
             )
     # First attempt a user install, then a system install
     try:
@@ -631,17 +530,23 @@ def install_requirement(
             echo=echo,
         )
     except OSError:
-        # If a user installation failed, attempt a system installation
-        run(
-            (
-                f"{quote(sys.executable)} -m pip install "
-                f"{quote(requirement_string)}"
-            ),
-            echo=echo,
-        )
+        try:
+            # If a user installation failed, attempt a system installation
+            run(
+                (
+                    f"{quote(sys.executable)} -m pip install "
+                    f"{quote(requirement_string)}"
+                ),
+                echo=echo,
+            )
+        except OSError as error:
+            append_exception_text(
+                error, f"Could not install {name} from {requirement_string}"
+            )
+            raise error
     # Refresh the metadata
     if distribution and editable:
-        setup_egg_info(distribution.location)
+        setup_dist_egg_info(distribution.location)
     else:
         refresh_working_set()
 
@@ -649,6 +554,8 @@ def install_requirement(
 def _get_pkg_requirement_distribution(
     requirement: pkg_resources.Requirement, name: str, reinstall: bool = True
 ) -> Optional[pkg_resources.Distribution]:
+    if name in _BUILTIN_DISTRIBUTION_NAMES:
+        return None
     try:
         return get_installed_distributions()[name]
     except KeyError:
@@ -658,11 +565,8 @@ def _get_pkg_requirement_distribution(
             f'The required distribution "{name}" was not installed, '
             "attempting to install it now..."
         )
-        try:
-            # Attempt to install the requirement...
-            install_requirement(requirement)
-        except OSError:
-            return None
+        # Attempt to install the requirement...
+        install_requirement(requirement)
         refresh_working_set()
         return _get_pkg_requirement_distribution(
             requirement, name, reinstall=False
@@ -678,6 +582,9 @@ def _iter_requirement_names(
     extras: Set[str] = set(map(normalize_name, requirement.extras))
     if name in exclude:
         return ()
+    # Ensure we don't follow the same requirement again, causing cyclic
+    # recursion
+    exclude.add(name)
     distribution: Optional[
         pkg_resources.Distribution
     ] = _get_pkg_requirement_distribution(requirement, name)
@@ -685,33 +592,35 @@ def _iter_requirement_names(
         return ()
     # Ensure requirements are up-to-date
     if _distribution_is_editable(distribution):
-        _setup_egg_info(distribution.location)
+        setup_dist_egg_info(distribution.location)
     requirements: List[pkg_resources.Requirement] = distribution.requires(
         extras=tuple(sorted(extras))
     )
+    lateral_exclude: Set[str] = set()
 
     def iter_requirement_names_(
         requirement_: pkg_resources.Requirement,
     ) -> Iterable[str]:
         return _iter_requirement_names(
             requirement_,
-            exclude - {_get_pkg_requirement_name(requirement_)},
-            recursive,
+            exclude=(
+                exclude
+                | (lateral_exclude - {_get_pkg_requirement_name(requirement_)})
+            ),
+            recursive=recursive,
         )
 
     def not_excluded(name: str) -> bool:
         if name not in exclude:
             # Add this to the exclusions
-            exclude.add(name)
+            lateral_exclude.add(name)
             return True
         return False
 
-    requirement_names: Iterable[str] = filter(
-        not_excluded, map(_get_pkg_requirement_name, requirements)
-    )
     if recursive:
         requirement_names = chain(
-            requirement_names, *map(iter_requirement_names_, requirements)
+            filter(not_excluded, map(_get_pkg_requirement_name, requirements)),
+            *map(iter_requirement_names_, requirements),
         )
     return requirement_names
 
@@ -786,7 +695,7 @@ def get_requirements_required_distribution_names(
 def iter_distribution_location_file_paths(location: str) -> Iterable[str]:
     location = os.path.abspath(location)
     name: str = get_setup_distribution_name(location)
-    setup_egg_info(location)
+    setup_dist_egg_info(location)
     metadata_path: str = os.path.join(
         location, f"{pkg_resources.to_filename(name)}.egg-info"
     )
